@@ -1,9 +1,12 @@
+from pathlib import Path
 from typing import Dict
 
 import black
-from datamodel_code_generator import PythonVersion
+import toml
 from isort import SortImports
 
+from datamodel_code_generator import PythonVersion
+
 BLACK_PYTHON_VERSION: Dict[PythonVersion, black.TargetVersion] = {
     PythonVersion.PY_36: black.TargetVersion.PY36,
     PythonVersion.PY_37: black.TargetVersion.PY37,
@@ -12,17 +15,29 @@ BLACK_PYTHON_VERSION: Dict[PythonVersion, black.TargetVersion] = {
 
 
 def format_code(code: str, python_version: PythonVersion) -> str:
+
+    code = apply_isort(code)
     code = apply_black(code, python_version)
-    return apply_isort(code)
+    return code
 
 
 def apply_black(code: str, python_version: PythonVersion) -> str:
 
+    root = black.find_project_root((Path().resolve(),))
+    path = root / "pyproject.toml"
+    if path.is_file():
+        value = str(path)
+        pyproject_toml = toml.load(value)
+        config = pyproject_toml.get("tool", {}).get("black", {})
+    else:
+        config = {}
+
     return black.format_str(
         code,
         mode=black.FileMode(
             target_versions={BLACK_PYTHON_VERSION[python_version]},
-            string_normalization=False,
+            line_length=config.get("line-length", black.DEFAULT_LINE_LENGTH),
+            string_normalization=not config.get("skip-string-normalization", True),
         ),
     )
 
