@@ -1,16 +1,17 @@
 from enum import Enum, auto
 from typing import Any, Dict, List, Optional
 
+from pydantic import BaseModel
+
 from datamodel_code_generator import PythonVersion
 from datamodel_code_generator.imports import Import
-from pydantic import BaseModel
 
 
 class DataType(BaseModel):
     type: str
     is_func: bool = False
     kwargs: Optional[Dict[str, Any]]
-    import_: Optional[Import]
+    imports_: Optional[List[Import]]
     python_version: PythonVersion = PythonVersion.PY_37
     unresolved_types: List[str] = []
     ref: bool = False
@@ -69,3 +70,4 @@ class Types(Enum):
     ipv4 = auto()
     ipv6 = auto()
     boolean = auto()
+    object = auto()
