@@ -3,20 +3,22 @@ from tempfile import NamedTemporaryFile
 from typing import List, Optional
 
 import pytest
+
 from datamodel_code_generator import PythonVersion
 from datamodel_code_generator.imports import Import
 from datamodel_code_generator.model.base import TemplateBase
 from datamodel_code_generator.model.pydantic import BaseModel, CustomRootType
-from datamodel_code_generator.parser.base import DataType, JsonSchemaObject
-from datamodel_code_generator.parser.openapi import OpenAPIParser, dump_templates
+from datamodel_code_generator.parser.base import DataType, dump_templates
+from datamodel_code_generator.parser.jsonschema import JsonSchemaObject
+from datamodel_code_generator.parser.openapi import OpenAPIParser
 
-DATA_PATH: Path = Path(__file__).parents[1] / 'data'
+DATA_PATH: Path = Path(__file__).parents[1] / 'data' / 'openapi'
 
 
 class A(TemplateBase):
     def __init__(self, filename: str, data: str):
         self._data = data
-        super().__init__(filename)
+        super().__init__(Path(filename))
 
     def render(self) -> str:
         return self._data
@@ -38,7 +40,7 @@ class A(TemplateBase):
         ('string', 'date-time', 'datetime', 'datetime', 'datetime'),
         ('string', 'password', 'SecretStr', 'pydantic', 'SecretStr'),
         ('string', 'email', 'EmailStr', 'pydantic', 'EmailStr'),
-        ('string', 'uri', 'UrlStr', 'pydantic', 'UrlStr'),
+        ('string', 'uri', 'AnyUrl', 'pydantic', 'AnyUrl'),
         ('string', 'uuid', 'UUID', 'uuid', 'UUID'),
         ('string', 'uuid1', 'UUID1', 'pydantic', 'UUID1'),
         ('string', 'uuid2', 'UUID2', 'pydantic', 'UUID2'),
@@ -58,7 +60,18 @@ def test_get_data_type(schema_type, schema_format, result_type, from_, import_):
     parser = OpenAPIParser(BaseModel, CustomRootType)
     assert parser.get_data_type(
         JsonSchemaObject(type=schema_type, format=schema_format)
-    ) == DataType(type=result_type, imports_=imports_)
+    ) == [DataType(type=result_type, imports_=imports_)]
+
+
+@pytest.mark.parametrize(
+    'schema_types,result_types',
+    [(['integer', 'number'], ['int', 'float']), (['integer', 'null'], ['int']),],
+)
+def test_get_data_type_array(schema_types, result_types):
+    parser = OpenAPIParser(BaseModel, CustomRootType)
+    assert parser.get_data_type(JsonSchemaObject(type=schema_types)) == [
+        DataType(type=r) for r in result_types
+    ]
 
 
 def test_get_data_type_invalid_obj():
@@ -239,7 +252,7 @@ def test_parse_root_type(source_obj, generated_classes):
 
 from typing import List, Optional
 
-from pydantic import BaseModel, UrlStr
+from pydantic import AnyUrl, BaseModel
 
 
 class Pet(BaseModel):
@@ -278,8 +291,8 @@ class Error(BaseModel):
 class api(BaseModel):
     apiKey: Optional[str] = None
     apiVersionNumber: Optional[str] = None
-    apiUrl: Optional[UrlStr] = None
-    apiDocumentationUrl: Optional[UrlStr] = None
+    apiUrl: Optional[AnyUrl] = None
+    apiDocumentationUrl: Optional[AnyUrl] = None
 
 
 class apis(BaseModel):
@@ -334,8 +347,8 @@ class Error(BaseModel):
 class api(BaseModel):
     apiKey: Optional[str] = None
     apiVersionNumber: Optional[str] = None
-    apiUrl: Optional[UrlStr] = None
-    apiDocumentationUrl: Optional[UrlStr] = None
+    apiUrl: Optional[AnyUrl] = None
+    apiDocumentationUrl: Optional[AnyUrl] = None
 
 
 class apis(BaseModel):
@@ -354,7 +367,7 @@ class Result(BaseModel):
             True,
             False,
             None,
-            '''from pydantic import BaseModel, UrlStr
+            '''from pydantic import AnyUrl, BaseModel
 from typing import List, Optional
 from __future__ import annotations
 
@@ -395,8 +408,8 @@ class Error(BaseModel):
 class api(BaseModel):
     apiKey: Optional[str] = None
     apiVersionNumber: Optional[str] = None
-    apiUrl: Optional[UrlStr] = None
-    apiDocumentationUrl: Optional[UrlStr] = None
+    apiUrl: Optional[AnyUrl] = None
+    apiDocumentationUrl: Optional[AnyUrl] = None
 
 
 class apis(BaseModel):
@@ -418,7 +431,7 @@ class Result(BaseModel):
 
 from typing import List, Optional
 
-from pydantic import UrlStr
+from pydantic import AnyUrl
 
 from custom_module import Base
 
@@ -459,8 +472,8 @@ class Error(Base):
 class api(Base):
     apiKey: Optional[str] = None
     apiVersionNumber: Optional[str] = None
-    apiUrl: Optional[UrlStr] = None
-    apiDocumentationUrl: Optional[UrlStr] = None
+    apiUrl: Optional[AnyUrl] = None
+    apiDocumentationUrl: Optional[AnyUrl] = None
 
 
 class apis(Base):
@@ -481,7 +494,7 @@ def test_openapi_parser_parse(with_import, format_, base_class, result):
     parser = OpenAPIParser(
         BaseModel,
         CustomRootType,
-        filename=str(DATA_PATH / 'api.yaml'),
+        text=Path(DATA_PATH / 'api.yaml').read_text(),
         base_class=base_class,
     )
     assert parser.parse(with_import=with_import, format_=format_) == result
@@ -510,7 +523,9 @@ def test_parse_root_type(source_obj, generated_classes):
 
 def test_openapi_parser_parse_duplicate_models():
     parser = OpenAPIParser(
-        BaseModel, CustomRootType, filename=str(DATA_PATH / 'duplicate_models.yaml')
+        BaseModel,
+        CustomRootType,
+        text=Path(DATA_PATH / 'duplicate_models.yaml').read_text(),
     )
     assert (
         parser.parse()
@@ -576,7 +591,9 @@ class DuplicateObject3(BaseModel):
 
 def test_openapi_parser_parse_resolved_models():
     parser = OpenAPIParser(
-        BaseModel, CustomRootType, filename=str(DATA_PATH / 'resolved_models.yaml')
+        BaseModel,
+        CustomRootType,
+        text=Path(DATA_PATH / 'resolved_models.yaml').read_text(),
     )
     assert (
         parser.parse()
@@ -610,7 +627,9 @@ class Resolved(BaseModel):
 
 def test_openapi_parser_parse_lazy_resolved_models():
     parser = OpenAPIParser(
-        BaseModel, CustomRootType, filename=str(DATA_PATH / 'lazy_resolved_models.yaml')
+        BaseModel,
+        CustomRootType,
+        text=Path(DATA_PATH / 'lazy_resolved_models.yaml').read_text(),
     )
     assert (
         parser.parse()
@@ -654,7 +673,7 @@ class Results(BaseModel):
 
 def test_openapi_parser_parse_enum_models():
     parser = OpenAPIParser(
-        BaseModel, CustomRootType, filename=str(DATA_PATH / 'enum_models.yaml')
+        BaseModel, CustomRootType, text=Path(DATA_PATH / 'enum_models.yaml').read_text()
     )
     assert (
         parser.parse()
@@ -698,13 +717,19 @@ class EnumRoot(Enum):
 class IntEnum(Enum):
     number_1 = 1
     number_2 = 2
+
+
+class AliasEnum(Enum):
+    a = 1
+    b = 2
+    c = 3
 '''
     )
 
     parser = OpenAPIParser(
         BaseModel,
         CustomRootType,
-        filename=str(DATA_PATH / 'enum_models.yaml'),
+        text=Path(DATA_PATH / 'enum_models.yaml').read_text(),
         target_python_version=PythonVersion.PY_36,
     )
     assert (
@@ -747,18 +772,25 @@ class EnumRoot(Enum):
 class IntEnum(Enum):
     number_1 = 1
     number_2 = 2
+
+
+class AliasEnum(Enum):
+    a = 1
+    b = 2
+    c = 3
 '''
     )
 
 
 def test_openapi_parser_parse_anyof():
     parser = OpenAPIParser(
-        BaseModel, CustomRootType, filename=str(DATA_PATH / 'anyof.yaml')
+        BaseModel, CustomRootType, text=Path(DATA_PATH / 'anyof.yaml').read_text()
     )
     assert (
         parser.parse()
         == '''from __future__ import annotations
 
+from datetime import date
 from typing import List, Optional, Union
 
 from pydantic import BaseModel
@@ -794,6 +826,7 @@ class AnyOfobj(BaseModel):
 
 class AnyOfArrayItem(BaseModel):
     name: Optional[str] = None
+    birthday: Optional[date] = None
 
 
 class AnyOfArray(BaseModel):
@@ -809,15 +842,16 @@ class Error(BaseModel):
 
 def test_openapi_parser_parse_allof():
     parser = OpenAPIParser(
-        BaseModel, CustomRootType, filename=str(DATA_PATH / 'allof.yaml')
+        BaseModel, CustomRootType, text=Path(DATA_PATH / 'allof.yaml').read_text()
     )
     assert (
         parser.parse()
         == '''from __future__ import annotations
 
+from datetime import date, datetime
 from typing import List, Optional
 
-from pydantic import BaseModel
+from pydantic import BaseModel, conint
 
 
 class Pet(BaseModel):
@@ -839,6 +873,11 @@ class AllOfobj(BaseModel):
     number: Optional[str] = None
 
 
+class AllOfCombine(Pet):
+    birthdate: Optional[date] = None
+    size: Optional[conint(ge=1.0)] = None
+
+
 class AnyOfCombine(Pet, Car):
     age: Optional[str] = None
 
@@ -861,6 +900,7 @@ class AnyOfCombineInArray(BaseModel):
 
 class AnyOfCombineInRoot(Pet, Car):
     age: Optional[str] = None
+    birthdate: Optional[datetime] = None
 
 
 class Error(BaseModel):
@@ -872,7 +912,7 @@ class Error(BaseModel):
 
 def test_openapi_parser_parse_alias():
     parser = OpenAPIParser(
-        BaseModel, CustomRootType, filename=str(DATA_PATH / 'alias.yaml')
+        BaseModel, CustomRootType, text=Path(DATA_PATH / 'alias.yaml').read_text()
     )
     assert (
         parser.parse()
@@ -933,7 +973,7 @@ class Result(BaseModel):
 from __future__ import annotations
 
 from enum import Enum
-from typing import Optional
+from typing import Any, Dict, List, Optional, Union
 
 from pydantic import BaseModel
 
@@ -958,7 +998,7 @@ class User(BaseModel):
 
 
 class Event(BaseModel):
-    name: Optional[str] = None
+    name: Optional[Union[str, float, int, bool, Dict[str, Any], List[str]]] = None
 ''',
                 (
                     'collections.py',
@@ -967,7 +1007,7 @@ from __future__ import annotations
 
 from typing import List, Optional
 
-from pydantic import BaseModel, UrlStr
+from pydantic import AnyUrl, BaseModel
 
 from . import models
 
@@ -987,8 +1027,8 @@ class Rules(BaseModel):
 class api(BaseModel):
     apiKey: Optional[str] = None
     apiVersionNumber: Optional[str] = None
-    apiUrl: Optional[UrlStr] = None
-    apiDocumentationUrl: Optional[UrlStr] = None
+    apiUrl: Optional[AnyUrl] = None
+    apiDocumentationUrl: Optional[AnyUrl] = None
 
 
 class apis(BaseModel):
@@ -1025,7 +1065,235 @@ def test_openapi_parser_parse_modular(with_import, format_, base_class, result):
     parser = OpenAPIParser(
         BaseModel,
         CustomRootType,
-        filename=str(DATA_PATH / 'modular.yaml'),
+        text=Path(DATA_PATH / 'modular.yaml').read_text(),
+        base_class=base_class,
+    )
+    assert parser.parse(with_import=with_import, format_=format_) == result
+
+
+@pytest.mark.parametrize(
+    'with_import, format_, base_class, result',
+    [
+        (
+            True,
+            True,
+            None,
+            '''from __future__ import annotations
+
+from typing import List, Optional
+
+from pydantic import BaseModel, Extra
+
+
+class Pet(BaseModel):
+    id: int
+    name: str
+    tag: Optional[str] = None
+
+
+class Pets(BaseModel):
+    __root__: List[Pet]
+
+
+class User(BaseModel):
+    class Config:
+        extra = Extra.allow
+
+    id: int
+    name: str
+    tag: Optional[str] = None
+
+
+class Users(BaseModel):
+    __root__: List[User]
+
+
+class Id(BaseModel):
+    __root__: str
+
+
+class Rules(BaseModel):
+    __root__: List[str]
+
+
+class Error(BaseModel):
+    code: int
+    message: str
+
+
+class Event(BaseModel):
+    name: Optional[str] = None
+
+
+class Result(BaseModel):
+    event: Optional[Event] = None
+''',
+        ),
+        (
+            False,
+            True,
+            None,
+            '''class Pet(BaseModel):
+    id: int
+    name: str
+    tag: Optional[str] = None
+
+
+class Pets(BaseModel):
+    __root__: List[Pet]
+
+
+class User(BaseModel):
+    class Config:
+        extra = Extra.allow
+
+    id: int
+    name: str
+    tag: Optional[str] = None
+
+
+class Users(BaseModel):
+    __root__: List[User]
+
+
+class Id(BaseModel):
+    __root__: str
+
+
+class Rules(BaseModel):
+    __root__: List[str]
+
+
+class Error(BaseModel):
+    code: int
+    message: str
+
+
+class Event(BaseModel):
+    name: Optional[str] = None
+
+
+class Result(BaseModel):
+    event: Optional[Event] = None
+''',
+        ),
+        (
+            True,
+            False,
+            None,
+            '''from pydantic import BaseModel, Extra
+from typing import List, Optional
+from __future__ import annotations
+
+
+class Pet(BaseModel):
+    id: int
+    name: str
+    tag: Optional[str] = None
+
+
+class Pets(BaseModel):
+    __root__: List[Pet]
+
+
+class User(BaseModel):
+    class Config:
+        extra = Extra.allow
+    id: int
+    name: str
+    tag: Optional[str] = None
+
+
+class Users(BaseModel):
+    __root__: List[User]
+
+
+class Id(BaseModel):
+    __root__: str
+
+
+class Rules(BaseModel):
+    __root__: List[str]
+
+
+class Error(BaseModel):
+    code: int
+    message: str
+
+
+class Event(BaseModel):
+    name: Optional[str] = None
+
+
+class Result(BaseModel):
+    event: Optional[Event] = None''',
+        ),
+        (
+            True,
+            True,
+            'custom_module.Base',
+            '''from __future__ import annotations
+
+from typing import List, Optional
+
+from pydantic import Extra
+
+from custom_module import Base
+
+
+class Pet(Base):
+    id: int
+    name: str
+    tag: Optional[str] = None
+
+
+class Pets(Base):
+    __root__: List[Pet]
+
+
+class User(Base):
+    class Config:
+        extra = Extra.allow
+
+    id: int
+    name: str
+    tag: Optional[str] = None
+
+
+class Users(Base):
+    __root__: List[User]
+
+
+class Id(Base):
+    __root__: str
+
+
+class Rules(Base):
+    __root__: List[str]
+
+
+class Error(Base):
+    code: int
+    message: str
+
+
+class Event(Base):
+    name: Optional[str] = None
+
+
+class Result(Base):
+    event: Optional[Event] = None
+''',
+        ),
+    ],
+)
+def test_openapi_parser_parse_additional_properties(
+    with_import, format_, base_class, result
+):
+    parser = OpenAPIParser(
+        BaseModel,
+        CustomRootType,
+        text=Path(DATA_PATH / 'additional_properties.yaml').read_text(),
         base_class=base_class,
     )
     assert parser.parse(with_import=with_import, format_=format_) == result
